'use strict';









var _chalk = require('chalk');var _chalk2 = _interopRequireDefault(_chalk);
var _diff = require('diff');




var _constants = require('./constants.js');function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { default: obj };}
const DIFF_CONTEXT_DEFAULT = 5; /**
                                 * Copyright (c) 2014, Facebook, Inc. All rights reserved.
                                 *
                                 * This source code is licensed under the BSD-style license found in the
                                 * LICENSE file in the root directory of this source tree. An additional grant
                                 * of patent rights can be found in the PATENTS file in the same directory.
                                 *
                                 * 
                                 */










const getColor = (added, removed) =>
added ? _chalk2.default.red : removed ? _chalk2.default.green : _chalk2.default.dim;

const getBgColor = (added, removed) =>
added ? _chalk2.default.bgRed : removed ? _chalk2.default.bgGreen : _chalk2.default.dim;

const highlightTrailingWhitespace = (line, bgColor) =>
line.replace(/\s+$/, bgColor('$&'));

const getAnnotation = options =>
_chalk2.default.green('- ' + (options && options.aAnnotation || 'Expected')) +
'\n' +
_chalk2.default.red('+ ' + (options && options.bAnnotation || 'Received')) +
'\n\n';

const diffLines = (a, b) => {
  let isDifferent = false;
  return {
    diff: (0, _diff.diffLines)(a, b).
    map(part => {const
      added = part.added,removed = part.removed;
      if (part.added || part.removed) {
        isDifferent = true;
      }

      const lines = part.value.split('\n');
      const color = getColor(added, removed);
      const bgColor = getBgColor(added, removed);

      if (lines[lines.length - 1] === '') {
        lines.pop();
      }

      return lines.
      map(line => {
        const highlightedLine = highlightTrailingWhitespace(line, bgColor);
        const mark = color(part.added ? '+' : part.removed ? '-' : ' ');
        return mark + ' ' + color(highlightedLine) + '\n';
      }).
      join('');
    }).
    join('').
    trim(),
    isDifferent };

};

// Only show patch marks ("@@ ... @@") if the diff is big.
// To determine this, we need to compare either the original string (a) to
// `hunk.oldLines` or a new string to `hunk.newLines`.
// If the `oldLinesCount` is greater than `hunk.oldLines`
// we can be sure that at least 1 line has been "hidden".
const shouldShowPatchMarks = (hunk, oldLinesCount) =>
oldLinesCount > hunk.oldLines;

const createPatchMark = hunk => {
  const markOld = `-${hunk.oldStart},${hunk.oldLines}`;
  const markNew = `+${hunk.newStart},${hunk.newLines}`;
  return _chalk2.default.yellow(`@@ ${markOld} ${markNew} @@\n`);
};

const structuredPatch = (a, b, contextLines) => {
  const options = {
    context:
    typeof contextLines === 'number' && contextLines >= 0 ?
    contextLines :
    DIFF_CONTEXT_DEFAULT };

  let isDifferent = false;
  // Make sure the strings end with a newline.
  if (!a.endsWith('\n')) {
    a += '\n';
  }
  if (!b.endsWith('\n')) {
    b += '\n';
  }

  const oldLinesCount = (a.match(/\n/g) || []).length;

  return {
    diff: (0, _diff.structuredPatch)('', '', a, b, '', '', options).hunks.
    map(hunk => {
      const lines = hunk.lines.
      map(line => {
        const added = line[0] === '+';
        const removed = line[0] === '-';

        const color = getColor(added, removed);
        const bgColor = getBgColor(added, removed);

        const highlightedLine = highlightTrailingWhitespace(line, bgColor);
        return color(highlightedLine) + '\n';
      }).
      join('');

      isDifferent = true;
      return shouldShowPatchMarks(hunk, oldLinesCount) ?
      createPatchMark(hunk) + lines :
      lines;
    }).
    join('').
    trim(),
    isDifferent };

};

function diffStrings(a, b, options) {
  // `diff` uses the Myers LCS diff algorithm which runs in O(n+d^2) time
  // (where "d" is the edit distance) and can get very slow for large edit
  // distances. Mitigate the cost by switching to a lower-resolution diff
  // whenever linebreaks are involved.
  const result =
  options && options.expand === false ?
  structuredPatch(a, b, options && options.contextLines) :
  diffLines(a, b);

  if (result.isDifferent) {
    return getAnnotation(options) + result.diff;
  } else {
    return _constants.NO_DIFF_MESSAGE;
  }
}

module.exports = diffStrings;